unit NViewLib; {$D+}

interface

uses
  Windows, Graphics;

type
  TNVLType = (nvlOther, nvlJPG, nvlGIF,
    nvlJIF, nvlBMP, nvlDIB, nvlRLE, nvlTGA, nvlPCX);

function NVLGetType(FileName: string): TNVLType;
function NVLGetWidth: Integer;
function NVLGetHeight: Integer;
function NVLLoadJPG(const FileName: string; ShowProgress: Boolean): TBitmap;
function NVLLoadGIF(const FileName: string; ShowProgress: Boolean): TBitmap;
function NVLLoadImage(const FileName: string; ShowProgress: Boolean): TBitmap;

implementation

uses
  Classes, SysUtils;

{ A simple function to get FileName types }

function NVLGetType(FileName: string): TNVLType;
var  i: Byte;
begin
  Result := nvlOther;
  FileName := ExtractFileName(FileName);
  repeat
    i := Pos('.', FileName);
    if (i <> 0) then
      FileName := Copy(FileName, i+1, Length(FileName)-i);
  until (i = 0);
  for i := 1 to Length(FileName) do FileName[i] := UpCase(FileName[i]);
  if (FileName = 'JPG') then Result := nvlJPG;
  if (FileName = 'GIF') then Result := nvlGIF;
  if (FileName = 'JIF') then Result := nvlJIF;
  if (FileName = 'BMP') then Result := nvlBMP;
  if (FileName = 'DIB') then Result := nvlDIB;
  if (FileName = 'RLE') then Result := nvlRLE;
  if (FileName = 'TGA') then Result := nvlTGA;
  if (FileName = 'PCX') then Result := nvlPCX;
end;

{ Inform the user that the DLL is not loaded }

procedure DefaultWarning; far;
begin
  MessageBox(0, 'A required .DLL file, NVIEWLIB.DLL, was not found.',
    'Error Starting NVIEWLIB.DLL',
    MB_TASKMODAL or MB_ICONEXCLAMATION or MB_OK);
end;

function DefaultGet: Integer;
far; stdcall;
begin  DefaultWarning;  Result := 0;  end;

function DefaultLoad(FileName: PChar; ShowProgress: Boolean): HBitmap;
far; stdcall;
begin  DefaultWarning;  Result := 0;  end;

{ Virtual functions }

type
  TGetFunc = function: Integer; stdcall;
  TLoadFunc = function(FileName: PChar; ShowProgress: Boolean): HBitmap;
    stdcall;

const GetWidth: TGetFunc = DefaultGet;
const GetHeight: TGetFunc = DefaultGet;
const Load_JPG: TLoadFunc = DefaultLoad;
const Load_GIF: TLoadFunc = DefaultLoad;
const LoadImage: TLoadFunc = DefaultLoad;

{ Initialization procedures }

const
  NViewLibHandle: THANDLE = 0;

procedure DoneNViewLib; far;
begin
  if (NViewLibHandle >= 32) then FreeLibrary(NViewLibHandle);
  NViewLibHandle := 0;
end;

procedure InitNViewLib;
var  ErrMode: Word;
begin
  if (NViewLibHandle = 0) then
  begin
    ErrMode := SetErrorMode(SEM_NOOPENFILEERRORBOX);
    NViewLibHandle := LoadLibrary('NViewLib.dll');
    SetErrorMode(ErrMode);
    if (NViewLibHandle < 32) then NViewLibHandle := 0;
    if (NViewLibHandle >= 32) then
    begin
      GetWidth := GetProcAddress(NViewLibHandle, 'GetWidth');
      GetHeight := GetProcAddress(NViewLibHandle, 'GetHeight');
      Load_JPG := GetProcAddress(NViewLibHandle, 'Load_JPG');
      Load_GIF := GetProcAddress(NViewLibHandle, 'Load_GIF');
      LoadImage := GetProcAddress(NViewLibHandle, 'NViewLoadLib');
    end;
    AddExitProc(DoneNViewLib);
  end;
end;

{ Unit functions }

function NVLGetWidth: Integer;
begin  InitNViewLib;  Result := GetWidth;  end;

function NVLGetHeight: Integer;
begin  InitNViewLib;  Result := GetHeight;  end;

function LoadFile(const FileName: string; SP: Boolean; TYP: TNVLType): TBitmap;
var
  FN: PChar;
  BMP: Graphics.TBitmap;
  R: TRect;
begin
  Result := nil;
  if (FileName = '') then Exit;
  if (TYP = nvlOther) then Exit;

  InitNViewLib;
  GetMem(FN, Length(FileName)+1);
  StrPCopy(FN, FileName);
  { Use a temp bmp because MViewLib maintains only 1 handle! }
  BMP := Graphics.TBitmap.Create;
  try
    if (TYP = nvlJPG) then BMP.Handle := Load_JPG(FN, SP) else
      if (TYP = nvlGIF) then BMP.Handle := Load_GIF(FN, SP) else
        BMP.Handle := LoadImage(FN, SP);
  finally end;
  FreeMem(FN, Length(FileName)+1);

  if (BMP.Handle <> 0) then
  begin
    Result := Graphics.TBitmap.Create;
    try
      with Result do
      begin
        Width := BMP.Width;  Height := BMP.Height;
        R := Rect(0, 0, Width, Height);
        Canvas.CopyRect(R, BMP.Canvas, R);
      end
    finally
      DeleteObject(BMP.Handle);
    end;
  end;
  BMP.Free;
end;

function NVLLoadJPG(const FileName: string; ShowProgress: Boolean): TBitmap;
begin
  Result := LoadFile(FileName, ShowProgress, nvlJPG);
end;

function NVLLoadGIF(const FileName: string; ShowProgress: Boolean): TBitmap;
begin
  { <!-- Place GIF Licence Checking here --> }
  Result := LoadFile(FileName, ShowProgress, nvlGIF);
end;

function NVLLoadImage(const FileName: string; ShowProgress: Boolean): TBitmap;
begin
  Result := LoadFile(FileName, ShowProgress, NVLGetType(FileName));
end;

{ No 'initialization' needed }

end.
